// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//
//	test the AbsorptionDataServer class
//
#include <iostream>
#include <cassert>
#include "CalServer.h"
#include "PhysicalConstants.h"
#include "AbsorptionDataServer.h"

using namespace std;

namespace {
bool isClose(double a, double b)
{
    return fabs(a)/fabs(b)-1. < 1.e-5;
}
}

int main()
{
	//	setup the the Cal server
	const Xmm::Instrument instrument = Xmm::EMOS1;
	calServer.setInstrument(instrument);
	calServer.state()->ccd()->set(1);
	calServer.state()->node()->set(EMOScam::PRIMARY_NODE);

	//	get the AbsorptionDataServer
	AbsorptionDataServer *acds = 0; acds = calServer.getAtom(acds);
	vector<string> v = acds->materials();

	double e[] = { 100., 1000., 5000., 10000., 15000. },
		   f1v[] = { -5.87427, 12.9974, 14.4195, 14.2747, 14.088 },
		   f2v[] = { 4.60161, 1.06956, 0.794992, 0.213514, 0.095787 };

	for (unsigned n=0; n<v.size(); ++n) {
		const AbsCoefCurves &acc = acds->henke(v[n]);
		const AbsCoefCurve &f1 = acc.f1(),
						   &f2 = acc.f2();
		cout<<"absorption coefficients for "<<v[n]<<endl;
		for (unsigned i=0; i<NUM_OF(e); ++i) {
			cout<<"f1("<<e[i]<<" eV) = "<<f1(e[i])<<" - "
				<<"f2("<<e[i]<<" eV) = "<<f2(e[i])<<endl;
			if (v[n] == "Si")
				assert(isClose(f1(e[i]), f1v[i]) &&
					   isClose(f2(e[i]), f2v[i]));
		}
		const AtomicProperties &ap = acds->atomicData(v[n]);
		cout<<"atomic data\n";
		cout<<"atomic number    :  "<<ap.number()<<endl;
		cout<<"atomic weight    : "<<ap.weight()<<endl;
		cout<<"density [g/cm^3] : "<<ap.density()<<endl;
	}

#if 0
    const AbsCoefCurve &f2Si    = acds->henke("Si").f2(),
					   &f2O		= acds->henke("O").f2();
    double ASi     = acds->atomicData("Si").weight(),
		   AO	   = acds->atomicData("O").weight();
	double rhoSi   = acds->atomicData("Si").density() * 1.e-3;
	double rhoSiO2 = 2.64;
	double ASiO2   = ASi + 2. * AO;

	vector<double> eGrid = f2Si.abscissa();
	unsigned N = eGrid.size();

	for (unsigned n=0; n<N; ++n) {
        double eV = eGrid[n];                              // [eV]
        double f0 = PhysicalConstants::avogadro * 2. *      // [1/mol]
                    PhysicalConstants::r0 * 1.e2 *          // [cm]
                    PhysicalConstants::eVperAo/eV * 1.e-8;  // [cm]
		cout<<eV<<' '<<f0 * rhoSi/ASi * f2Si(eV)<<endl;
	}

	cout<<endl<<endl;
	for (unsigned n=0; n<N; ++n) {
        double eV = eGrid[n];                              // [eV]
        double f0 = PhysicalConstants::avogadro * 2. *      // [1/mol]
                    PhysicalConstants::r0 * 1.e2 *          // [cm]
                    PhysicalConstants::eVperAo/eV * 1.e-8;  // [cm]
		double a  = (rhoSiO2/ASiO2 * f2Si(eV) +
					2. * rhoSiO2/ASiO2 * f2O(eV))*f0;
		cout<<eV<<' '<<a<<endl;
	}
#endif

	return 0;
}
