// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//
//	test the cal
//
#include <iostream>
#include <cmath>
#include "CalServer.h"
#include "Xmm.h"
#include "EnergyCorrector.h"

using namespace std;

static bool isClose(double a, double b)
{
    //return fabs(fabs(a/b)-1.) < 1.e-5;
    return true;
}


int main()
{
	calServer.setInstrument(Xmm::EPN);
	calServer.state()->ccd()->set(EPNcam::CCD_1);
	calServer.state()->mode()->set(Xmm::PRIME_FULL_WINDOW);
	calServer.state()->accuracyLevel()->set(HIGH);
	calServer.state()->temperature()->set(163.);

	//
	//	high-accuracy gain correction
	//
	EnergyCorrector *gainCorrector = 0;
	gainCorrector = calServer.getAtom(gainCorrector);

	double timeVec[] = { 0., 0., 0., 0., 0. };
	short rawXVec[] = { 17, 18, 53, 61, 1 },
	  rawYVec[] = { 8, 8, 8, 24, 1 };
	int	  frameVec[] = { 1, 1, 1, 1, 1 };
	float eInVec[] = { 1245., 89., 15., 1256., 7242. };

	float eOutGainVec[] = { 1252.4, 89.0775, 15.2838, 1313.97, 205.773 },
	  eOutCtiVec[] = { 1256.8, 89.6232, 15.4146, 1326.7, 209.201 };
	  
	CalReal64Vector time(timeVec, NUM_OF(timeVec));
	CalInt16Vector rawX(rawXVec, NUM_OF(rawXVec)),
	  rawY(rawYVec, NUM_OF(rawYVec));
	CalInt32Vector frame(frameVec, NUM_OF(frameVec));

	CalReal32Vector eIn(eInVec, NUM_OF(eInVec)),
					eOut(0, NUM_OF(eInVec));

	gainCorrector->gainCorrect(time, rawX, rawY, eIn, eOut);

	for (unsigned n=0; n<eIn.size(); ++n) {
	  cout<<"gain: "<<time[n]<<" "<<rawX[n]<<" "<<rawY[n]<<" "
	      <<eIn[n]<<" "<<eOut[n]<<endl;
	  assert(isClose(eOut[n], eOutGainVec[n]));
	}

        cout<<"Additional corrector"<<endl;
        STime s(2017U);
        calServer.state()->exposureStartTime()->set(s);

	gainCorrector->additionalGainCtiCorrect(eOut, rawY, 165.0, true,
	                          false, true, true, true);
        cout<<"Additional corrector"<<endl;

	calServer.releaseAtom(gainCorrector);
	gainCorrector = 0;

	//
	//	high-accuracy CTI correction
	//
	CtiCorrector *ctiCorrector = 0;
	ctiCorrector = calServer.getAtom(ctiCorrector);

	for (unsigned n=0; n<eIn.size(); ++n)
		eIn[n] = eOut[n];

	ctiCorrector->ctiCorrect(time, rawX, rawY, frame, eIn, eOut);

	for (unsigned n=0; n<eIn.size(); ++n) {
		cout<<"CTI : "<<time[n]<<" "<<rawX[n]<<" "<<rawY[n]<<" "
			<<frame[n]<<" "
			<<eIn[n]<<" "<<eOut[n]<<endl;
		assert(isClose(eOut[n], eOutCtiVec[n]));
	}
	calServer.releaseAtom(ctiCorrector);

        //
        //      pn spatial CTI correction
        //
        cout<<"Gain"<<endl;

	gainCorrector = calServer.getAtom(gainCorrector);

        for (unsigned n=0; n<eIn.size(); ++n)
                eIn[n] = eOut[n];

        gainCorrector->spatialCtiCorrect(eOut, rawX, rawY);

        for (unsigned n=0; n<eOut.size(); ++n) {
                cout<<"SPATIAL : "<<rawX[n]<<" "<<rawY[n]<<" "
                        <<eIn[n]<<" "<<eOut[n]<<endl;
        }

        calServer.releaseAtom(gainCorrector);
	gainCorrector = 0;

	calServer.state()->ccd()->set(EPNcam::CCD_2);

	gainCorrector = calServer.getAtom(gainCorrector);

        gainCorrector->spatialCtiCorrect(eOut, rawX, rawY);

        for (unsigned n=0; n<eOut.size(); ++n) {
                cout<<"SPATIAL : "<<rawX[n]<<" "<<rawY[n]<<" "
                        <<eIn[n]<<" "<<eOut[n]<<endl;
        }

        calServer.releaseAtom(gainCorrector);
	gainCorrector = 0;

        //
        //      Sanders et al. PN spatial CTI correction
        //
        // Set the processing epoch
        short rev=1300;
        calServer.state()->exposureRevolution()->set(rev);

        gainCorrector = calServer.getAtom(gainCorrector);
        calServer.state()->ccd()->set(EPNcam::CCD_6);

        for (unsigned n=0; n<eIn.size(); ++n)
                eIn[n] = eOut[n];

        gainCorrector->spatialSD20SetElement("CU");
        gainCorrector->spatialCtiCorrectSD20(eOut, rawX, rawY);

        for (unsigned n=0; n<eOut.size(); ++n) {
                cout<<"SPATIAL_SD : "<<rawX[n]<<" "<<rawY[n]<<" "
                        <<eIn[n]<<" "<<eOut[n]<<endl;
        }

        calServer.releaseAtom(gainCorrector);
	gainCorrector = 0;
        
        // Rev 4200
        rev=101;
        calServer.state()->exposureRevolution()->set(rev);

        gainCorrector = calServer.getAtom(gainCorrector);
        calServer.state()->ccd()->set(EPNcam::CCD_1);

        for (unsigned n=0; n<eIn.size(); ++n)
                eIn[n] = eOut[n];

        // Try both elements
        gainCorrector->spatialSD20SetElement("MN");
        gainCorrector->spatialCtiCorrectSD20(eOut, rawX, rawY);

        for (unsigned n=0; n<eOut.size(); ++n) {
                cout<<"SPATIAL_SD : "<<rawX[n]<<" "<<rawY[n]<<" "
                        <<eIn[n]<<" "<<eOut[n]<<endl;
        }
        for (unsigned n=0; n<eIn.size(); ++n)
                eIn[n] = eOut[n];

        // Try both elements
        gainCorrector->spatialSD20SetElement("CU");
        gainCorrector->spatialCtiCorrectSD20(eOut, rawX, rawY);

        for (unsigned n=0; n<eOut.size(); ++n) {
                cout<<"SPATIAL_SD : "<<rawX[n]<<" "<<rawY[n]<<" "
                        <<eIn[n]<<" "<<eOut[n]<<endl;
        }
        //      rate dependent PHA parameter
        //
        calServer.setInstrument(Xmm::EPN);
//	calServer.state()->ccd()->set(EPNcam::CCD_2);

        calServer.state()->ccd()->set(EPNcam::CCD_2);
        cout<<"Testing RDPHA"<<endl;

        gainCorrector = calServer.getAtom(gainCorrector);
        calServer.state()->mode()->set(Xmm::FAST_TIMING);

        double Ne=400.0;
        bool withGainTiming=true;
        vector<double> rdp = 
                   gainCorrector->getRateDepPhaCoeffs(Ne, withGainTiming);
        cout<<"Rate dependent PHA coefficients (TIMING): "<<rdp[0]<<" "
            <<rdp[1]<<" "<<rdp[2]<<" " <<rdp[3]<<" "<<rdp[4]<<" "
             <<rdp[5]<<" "<<rdp[6]<<" " <<rdp[7]<<" "<<rdp[8]<<" "<<endl;

        calServer.state()->mode()->set(Xmm::FAST_BURST);
        rdp = gainCorrector->getRateDepPhaCoeffs(Ne, withGainTiming);
        cout<<"Rate dependent PHA coefficients (BURST): "<<rdp[0]<<" "
            <<rdp[1]<<" "<<rdp[2]<<" " <<rdp[3]<<" "<<rdp[4]<<" "
             <<rdp[5]<<" "<<rdp[6]<<" " <<rdp[7]<<" "<<rdp[8]<<" "<<endl;

        calServer.releaseAtom(gainCorrector);
	gainCorrector = 0;

        // BURST mode should produce an error
        calServer.state()->mode()->set(Xmm::FAST_BURST);
//rate-dep gain coeffs:0.098 0.047 0.847
//CTI : 0 17 ode()->set(Xmm::FAST_BURST);
        gainCorrector = calServer.getAtom(gainCorrector);

        rdp = gainCorrector->getRateDepPhaCoeffs(Ne, true);
        cout<<"Rate dependent PHA coefficients: "<<rdp[0]<<" "
            <<rdp[1]<<" "<<rdp[2]<<endl;

        calServer.releaseAtom(gainCorrector);
	gainCorrector = 0;

        
	//
	//	low-accuracy gain/CTI
	//
	float eInVec2[] = { 1245., 89., 15., 1256., 199. };

	CalReal32Vector eIn2(eInVec2, NUM_OF(eInVec)),
					eOut2(0, NUM_OF(eInVec));

	float eOutGainVec2[] = { 1208.46, 86.0485, 15.1155, 1258.75, 199.629 },
		  eOutCtiVec2[] = { 1214.75, 86.5035, 15.1995, 1279.09, 202.487 };

	calServer.state()->accuracyLevel()->set(LOW);
	gainCorrector = calServer.getAtom(gainCorrector);

	gainCorrector->gainCorrect(time, rawX, rawY, eIn2, eOut2);
	for (unsigned n=0; n<eIn.size(); ++n) {
		cout<<"gain: "<<time[n]<<" "<<rawX[n]<<" "<<" "
			<<eIn2[n]<<" "<<eOut2[n]<<endl;
		assert(isClose(eOut2[n], eOutGainVec2[n]));
	}

       // Test the rate-dependent gain correction
	calServer.state()->mode()->set(Xmm::PRIME_FULL_WINDOW);
        vector<float> rdc = gainCorrector->getRateDepGainCoeffs();
        cout<<"rate-dep gain coeffs:"<<rdc[0]<<" "<<rdc[1]<<" "
            <<rdc[2]<<" "<<endl;

	gainCorrector = 0;
        calServer.releaseAtom(gainCorrector);
      /*
        calServer.releaseAtom(gainCorrector);
        calServer.state()->mode()->set(Xmm::FAST_TIMING);
        gainCorrector = calServer.getAtom(gainCorrector);
        rdc = gainCorrector->getRateDepGainCoeffs();
        cout<<"rate-dep gain coeffs TIMING:"<<rdc[0]<<" "<<rdc[1]<<" "
            <<rdc[2]<<" "<<endl;
      */

	ctiCorrector = calServer.getAtom(ctiCorrector);
	for (unsigned n=0; n<eIn.size(); ++n)
		eIn2[n] = eOut2[n];

	ctiCorrector->ctiCorrect(time, rawX, rawY, frame, eIn2, eOut2);

	for (unsigned n=0; n<eIn.size(); ++n) {
		cout<<"CTI : "<<time[n]<<" "<<rawX[n]<<" "<<rawY[n]<<" "
			<<frame[n]<<" "
			<<eIn2[n]<<" "<<eOut2[n]<<endl;
		assert(isClose(eOut2[n], eOutCtiVec2[n]));
	}

	return 0;
}







