! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
!
!   psftest.f90  --- f90 program to test the PSF-related CAL routines
!
!   DESCRIPTION:
!       This tests a few aspects of the PSF-related CAL routines
!
program psftest

use cal             ! use the Calibration Access Layer
use dal
use errorhandling

    implicit none
    logical                        :: isClose
    integer, dimension(0:2)        :: insVec = (/EMOS1, EMOS2, EPN/)
    integer, dimension(0:3)        :: accVec = (/ACCURACY_LOW, &
                                                 ACCURACY_MEDIUM, &
                                                 ACCURACY_EXTENDED, &
                                                 ACCURACY_HIGH/)
    integer                        :: ins, acc, Ei, nx, ny, iter
    real(double), dimension(0:3)   :: Evec = (/100, 1500, 4500, 7000/)
    real(double), dimension(0:26)  :: onval = (/ &
    0.4091759723441300, &
    0.3720914778616852, &
    0.3209737093831113, &
    0.5382853696743647, &
    0.4057884232865440, &
    0.2676861379985456, &
    0.3492290574527561, &
    0.3492290574527561, &
    0.3492290574527561, &
    0.4093759285804727, &
    0.3722808465156883, &
    0.3211405592937853, &
    0.5382853696743647, &
    0.4057884232865440, &
    0.2676861379985456, &
    0.2942574998318281, &
    0.2942574998318281, &
    0.2942574998318281, &
    0.4094092624793676, &
    0.3723124157836069, &
    0.3211683747211920, &
    0.5382853696743647, &
    0.4057884232865440, &
    0.2676861379985456, &
    0.2942628565316449, &
    0.2942628565316449, &
    0.2942628565316449/)
    real(double), dimension(0:26)  :: eeval = (/ &
    0.3610813334705640, &
    0.3610813334705640, &
    0.3610813334705640, &
    2.4824341676101280, &
    3.7462188347571024, &
    5.0100035019040767, &
    2.3470286675586665, &
    2.3470286675586665, &
    2.3470286675586665, &
    0.3610813334705640, &
    0.3610813334705640, &
    0.3610813334705640, &
    2.4824341676101280, &
    3.7462188347571024, &
    5.0100035019040767, &
    3.6559485013894615, &
    3.6559485013894615, &
    3.6559485013894615, &
    0.3610813334705640, &
    0.3610813334705640, &
    0.3610813334705640, &
    2.4824341676101280, &
    3.7462188347571024, &
    5.0100035019040767, &
    3.6559485013894615, &
    3.6559485013894615, &
    3.6559485013894615/)

    real(double)                   :: E, ee, onAxisVal
    type(PsfT)                     :: psf
    type(PsfValidityRangesT)       :: valRange
    type(PsfBinSizeT)              :: binSize
    type(PsfWindowT)               :: window
    logical                        :: binSizeChangeable
    real(single), dimension(:,:), pointer :: map
    real(single)                   :: total, bigtot
    real(double)                   :: posang

    iter = 0
    do ins=2, 2
        do acc=2, 2
            call CAL_setState(instrument=insVec(ins), &
                              accuracyLevel=accVec(acc))
            do Ei=0, size(Evec-1)-1
               E = Evec(Ei)
               write(*, *) 'Retrieving PSF for ', instId(insVec(ins)), &
                           ' acc=', acc, 'at E=', E, ' eV'
               psf = CAL_getPSF(E, 0._double, 0._double)

             ! Test the position angle setting
               posang=0.4_double
               call CAL_psfSetPosAngle(psf, posang)

               valRange = CAL_psfValidityRanges()
               write(*, *) 'validity ranges: '
               write(*, *) 'eMin         : ', valRange%eMin
               write(*, *) 'eMax         : ', valRange%eMax
               write(*, *) 'thetaMin     : ', valRange%thetaMin
               write(*, *) 'thetaMax     : ', valRange%thetaMax
               write(*, *) 'phiMin       : ', valRange%phiMin
               write(*, *) 'phiMax       : ', valRange%phiMax
               onAxisVal = CAL_psfPointValue(psf, 0._double, 0._double)
               write(*, *) 'value on-axis: ', onAxisVal
               if (.not.isClose(onAxisVal, onval(iter))) &
                   ! call error('psftest', 'incorrect on-axis PSF value')
               binSize = CAL_psfBinSize(psf)
               binSizeChangeable = CAL_psfBinSizeChangeable(psf);
               write(*, *) 'bin sizes    : ', binSize%binX, binSize%binY, &
                                              binSizeChangeable
               if (binSizeChangeable) then
                  !binSize%binX = 0.14898/4   ! 4.1 arcsecs
                  binSize%binX = 0.036337   ! 4.1 arcsecs
                  !binSize%binX = 0.15988   ! 4.4 arcsecs
                  !binSize%binY = 0.14898 
                  !binSize%binX = 0.04    ! 1.1 arcsec
                  binSize%binY = 0.04
                  binSize%binY = 0.036337
                 !  binSize%binY = 29.796 ! PN full CCD
                  !binSize%binY = 23.982   ! MOS Full CCD
                  !binSize%binY = 23.982   ! MOS Full CCD
                  call CAL_psfSetBinSize(psf, binSize)
                  binSize = CAL_psfBinSize(psf)
               endif
               write(*, *) 'bin sizes    : ', binSize%binX, binSize%binY, &
                                              binSizeChangeable
               !window%halfY = 11.991_double
               !window%halfY = .11991_double
               window%halfY = 14.898_double
               window%halfX = binSize%binX * 300.0_double
               !window%halfY = binSize%binY * 10.0_double
               write(*, *) 'image centered on PSF barycenter - ', &
                            'extent: ', window%halfX, 'x', window%halfY, 'mm'
               call CAL_psfImage(psf, window, map)
               write(*,*)' Map size: ',size(map, 1),size(map, 2)
            total=0.0
            bigtot=0.0
               do nx=0, size(map, 1)-1
                   !write(*, '(100F10.4, 1X)') &
                    !          (map(nx, ny)*10000.0, ny=0, size(map,2)-1)
                  !total=0.0
                  do ny=0, size(map,2)-1
                     total = total+map(nx, ny)*1000.0
                     bigtot=bigtot+map(nx, ny)
                  enddo

                  if (mod((nx+2),4) .eq. 0) then 
                     write(*, *)nx,total
                     total=0
                  endif

               enddo
               write(*, *) 'TOTAL: ',bigtot
               call CAL_releaseMemory(map)

               ee = CAL_psfEncircledEnergy(psf, .95_double)
               write(*, *) '95% of the energy contained in circle with ', &
                           'radius', ee, ' mm'
               if (.not.isClose(ee, eeval(iter))) &
        !           call error('psftest', 'incorrect EE value')
               iter = iter + 1
            enddo
        enddo
    enddo      

    call message(AppMsg, Verbose, "all tests passed")
end program

 
logical function isClose(value, trueval)
use types

    real(double), intent(in)    :: value, trueval
    real(double)                :: absval, abstrueval

    isClose = .true.
    if (value * trueval .lt. 0.) isClose = .false.
    absval = abs(value)
     abstrueval = abs(trueval)
    if (absval .lt. .9999 * abstrueval .or. &
        absval .gt. 1.001 * abstrueval) isClose = .false.
end function isClose
