// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	EqpbServer.h --- Cal atom to return swcx images
//
//	DESCRIPTION:
//
//	CHANGE HISTORY:
//		RDS, 2016-09-26: 1.0: - original
//
//	AUTHOR:
//		R. Saxton
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al., 
//
#ifndef Eqpb_H
#define Eqpb_H

#include "Ccf.h"
#include "CalAtomsBase.h"

#include <vector>

//
//	---( mode parameters )------------------------------------------------
//
/*
typedef struct {
            vector<unsigned int> quad;
            vector<string> obsid, expid, submode, filter, date, object;
            vector<float> total_t, clean_t, exposure, rate, rateerr;
            vector<float> rate1, rate1err, rate2, rate2err, rate3,
                      rate3err, rate4, rate4err, hard1, hard1err,
                      hard2, hard2err, hard3, hard3err;
            vector<vector<float> > specp0, spec00p0, specp4, spec00p4;
} QpbData;
*/
//
class EqpbServer : public CalAtom,
        public virtual CalChild<CalAtomCollection<EqpbServer> > { 
public:

	struct QpbData {
            vector<unsigned int> quad;
            vector<string> obsid, expid, submode, filter, date, object;
            vector<float> total_t, clean_t, exposure, rate, rateerr;
            vector<float> rate1, rate1err, rate2, rate2err, rate3,
                      rate3err, rate4, rate4err, hard1, hard1err,
                      hard2, hard2err, hard3, hard3err;
            vector<int> backscal, revolut;
            vector<vector<int> > specp0, specoop0, specp4, specoop4;
	};

	// Constructor
	EqpbServer(QpbData qpbData, 
                        CalAtomCollection<EqpbServer> *parent=0);

        const QpbData qpbData() const;

	static unsigned id() { return Ccf::QPB; };


private:

        QpbData _qpbData;

};


#endif

